#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/8/19'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'


class JITError(Exception):
    pass

class PreviewingError(JITError):
    pass





#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/4/27'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

from common import *
import re


class Check(object):

    def __init__(self):
        pass

    @staticmethod
    def previewing(members=None, set_=None):
        """
        :rtype : dict
        :param members: 合法对象的规则描述
        :param set_: 存放对象的集合
        :return: dict格式的检测结果
        """
        if members is None:
            members = []

        if set_ is None:
            set_ = {}

        result = dict()
        result['state'] = Common.exchange_state(20000)

        for item in members:
            if not isinstance(item, tuple):
                result['state'] = Common.exchange_state(41204)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], u'，收到 ',
                                                           type(item).__name__, u'，源自 ', str(item)])
                raise ji.PreviewingError(json.dumps(result))

            member_type = None
            member_range = None
            member_need = True

            if item.__len__() == 4:
                member_type = item[0]
                member_name = item[1]
                member_range = item[2]
                member_need = item[3]

            elif item.__len__() == 3:
                member_type = item[0]
                member_name = item[1]
                member_range = item[2]

            elif item.__len__() == 2:
                member_type = item[0]
                member_name = item[1]

            elif item.__len__() == 1:
                member_name = item[0]

            else:
                result['state'] = Common.exchange_state(41205)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], u'，收到 ',
                                                           str(item.__len__()), u'，源自 ', str(item)])
                raise ji.PreviewingError(json.dumps(result))

            if not isinstance(member_name, basestring):
                result['state'] = Common.exchange_state(41207)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], u'，收到 ',
                                                           type(member_name).__name__, u'，源自 ', str(item)])
                raise ji.PreviewingError(json.dumps(result))

            if member_name not in set_:
                if member_need:
                    result['state'] = Common.exchange_state(41201)
                    result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], str(member_name)])
                    raise ji.PreviewingError(json.dumps(result))

                else:
                    return result

            if member_type is not None:
                if isinstance(member_type, basestring):
                    if 0 == member_type.find('regex:'):
                        if re.match(member_type[6:], set_[member_name]) is None:
                            result['state'] = Common.exchange_state(41209)
                            raise ji.PreviewingError(json.dumps(result))

                elif not isinstance(set_[member_name], member_type):
                    result['state'] = Common.exchange_state(41202)
                    result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], u'，预期类型 ',
                                                               str(member_type), u'，收到 ',
                                                               type(set_[member_name]).__name__,
                                                               u'，源自字段 ', str(member_name)])
                    raise ji.PreviewingError(json.dumps(result))

            if member_range is not None:
                if isinstance(member_range, tuple):
                    if member_range.__len__() < 2:
                        result['state'] = Common.exchange_state(41206)
                        result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], u'，预期2个元素，收到 ',
                                                                   str(member_range.__len__()), u'，源自 ',
                                                                   str(member_range)])
                        raise ji.PreviewingError(json.dumps(result))

                    if member_type in [basestring, str, unicode, tuple, list, dict]:
                        me = set_[member_name].__len__()
                    else:
                        me = set_[member_name]

                    if not member_range[0] <= me <= member_range[1]:
                        result['state'] = Common.exchange_state(41203)
                        result['state']['sub']['zh-cn'] = ''.join([member_name, result['state']['sub']['zh-cn'],
                                                                   u'，预期取值范围 ', str(member_range), u'，收到 ',
                                                                   str(set_[member_name])])
                        raise ji.PreviewingError(json.dumps(result))

                elif isinstance(member_range, list):
                    if member_range.__len__() < 1:
                        result['state'] = Common.exchange_state(41206)
                        result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], u'，不可少于1个，收到 ',
                                                                   str(member_range.__len__()), u'，源自 ',
                                                                   str(member_range)])
                        raise ji.PreviewingError(json.dumps(result))

                    if set_[member_name] not in member_range:
                        result['state'] = Common.exchange_state(41203)
                        result['state']['sub']['zh-cn'] = ''.join([member_name, result['state']['sub']['zh-cn'],
                                                                   u'，预期取值范围 ', str(member_range), u'，收到 ',
                                                                   str(set_[member_name])])
                        raise ji.PreviewingError(json.dumps(result))

                else:
                    result['state'] = Common.exchange_state(41206)
                    result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], u'，不支持的类型 ',
                                                               type(member_range).__name__])
                    raise ji.PreviewingError(json.dumps(result))

        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/6/26'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'


from smtplib import SMTP
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage

"""
Reference: [
    https://docs.python.org/2/library/email.html,
    http://www.cnblogs.com/xiaowuyi/archive/2012/03/17/2404015.html,
    http://www.tutorialspoint.com/python/python_sending_email.htm
]
"""

class NetUtils(object):

    @staticmethod
    def smtp_init(host='', port=None, login_name='', password='', tls=False):
        if port is None:
            if tls:
                tls = 587
            else:
                tls = 25

        smtp_server = SMTP(host, port)
        if tls:
            smtp_server.starttls()
        smtp_server.login(login_name, password)
        return smtp_server

    @staticmethod
    def send_mail(smtp_server=None, sender='', receivers=None, title='', message='', html=False, attachments=None):
        """
        :rtype : dict
        :param smtp_server: 一个已建立好连接的smtp实例
        :param sender: 发送者邮件地址('sender@jimit.cc')
        :param receivers: 收件者们的地址(['user1@jimit.cc', 'user2@jimit.cc'])
        :param title: 邮件标题('这是一个测试邮件')
        :param message: 邮件内容('Hello!')
        :param html: 邮件内容类型是否为html格式
        :param attachments: 附件列表，类型为MIMEImage
        :return: 返回发送结果
        """
        if receivers is None:
            receivers = []

        if attachments is None:
            attachments = []

        if smtp_server is None:
            raise ValueError('smtp_server must is not None.')

        mail = MIMEMultipart()
        mail['Subject'] = title
        mail['From'] = sender
        mail['To'] = ','.join(receivers)

        if html:
            message = MIMEText(message, "html", 'utf8')
        else:
            message = MIMEText(message, "plain", 'utf8')

        mail.attach(message)
        for attachment in attachments:
            mail.attach(attachment)

        return smtp_server.sendmail(sender, receivers, mail.as_string())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '16/1/24'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'


import time
import uuid
import os
import thread
from jimit import JITime


class TransactionSerialNumber(object):

    def __init__(self):
        # YYYYMMDDHHmmSS
        self.date_time = JITime.now_date_time(date_separator='', time_separator='', dt_separator='')
        # MAC 地址的后两段的十进制值, 如 ff:ff 为 65535
        self.mac_number = '{:05d}'.format(int(uuid.uuid1().__str__()[32:], 16))
        self.pid = '{:05d}'.format(os.getpid())
        self.tsn_counter = 0

    def increment_tsn_counter(self, number=1):
        self.tsn_counter += number

    # clicking 滴答,如演奏的拍子一样
    def clicking(self):
        self.date_time = JITime.now_date_time(date_separator='', time_separator='', dt_separator='')
        self.tsn_counter = 0

    def generate_tsn(self, separator=''):
        self.increment_tsn_counter(1)
        return separator.join([self.date_time, self.mac_number, self.pid, '{:05d}'.format(self.tsn_counter)])

    def watch_tsn(self):
        while True:
            time.sleep(1)
            self.clicking()

    def launch(self):
        thread.start_new_thread(self.watch_tsn, ())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '16/1/17'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

import time
import os
import json
import thread
import signal


class KeepRead(object):
    """
    配置文件默认路径
        /etc/monitor_log.conf
        通过修改参数 KeepRead.config_path 来替换

    游标持久化默认存储路径
        /var/lib/misc/cursor.ml
        通过修改参数 KeepRead.cursor_path 来替换

    用法:
        设计过滤器方法,输入的参数仅有一个,即每次调用时所读取的行
        @staticmethod
        def filtrator(line=None):
            print line

        首先指定过滤器
        KeepRead.filtrator = filtrator
        启动
        KeepRead.launch()
    """
    # Example:
    # {"/var/log/messages": 2341, "/var/log/nginx/error_log": 3521}
    cursor = {}
    # Example:
    # [
    #   {"path": "/var/log/messages", "start_position": 0},
    #   {"path": "/var/log/nginx/error_log", "start_position": 0}
    # ]
    config = []
    log_file_ino = {}
    config_path = '/etc/monitor_log.conf'
    cursor_path = '/var/lib/misc/cursor.ml'
    exit_flag = False
    thread_counter = 0
    filtrator = None

    def __init__(self):
        self.log_path = ''
        self.line = ''

    @classmethod
    def save_cursor(cls):
        dirname = os.path.dirname(cls.cursor_path)
        if not os.path.exists(dirname):
            os.makedirs(dirname)

        with open(cls.cursor_path, 'w') as f:
            f.write(json.dumps(cls.cursor))

    @classmethod
    def restore_cursor(cls):
        if os.path.exists(cls.cursor_path):
            with open(cls.cursor_path) as f:
                cls.cursor = json.load(f)

    @classmethod
    def load_config(cls):
        if os.path.exists(cls.config_path):
            with open(cls.config_path, 'r') as f:
                cls.config = json.load(f)

    @classmethod
    def set_cursor(cls, log_path=None, offset=0):
        cls.cursor[log_path] = offset

    @classmethod
    def get_cursor(cls, log_path=None):
        return cls.cursor[log_path]

    @classmethod
    def increment_thread_counter(cls, number=1):
        cls.thread_counter += number

    @classmethod
    def get_exit_flag(cls):
        return cls.exit_flag

    @classmethod
    # ino for index node number
    def watch_ino(cls):
        cls.increment_thread_counter(1)
        while True:
            if cls.get_exit_flag():
                cls.increment_thread_counter(-1)
                return

            try:
                for item in KeepRead.config:
                    st_ino = os.stat(item['path']).st_ino
                    if item['path'] not in cls.log_file_ino:
                        cls.log_file_ino[item['path']] = dict()
                        cls.log_file_ino[item['path']]['ino_changed'] = False
                        cls.log_file_ino[item['path']]['last'] = st_ino

                    if cls.log_file_ino[item['path']]['last'] != st_ino:
                        cls.log_file_ino[item['path']]['ino_changed'] = True
                        cls.log_file_ino[item['path']]['last'] = st_ino

            except OSError:
                pass
            except:
                cls.increment_thread_counter(-1)
                raise

            time.sleep(1)

    @classmethod
    def get_ino_changed(cls, path=None):
        if path not in cls.log_file_ino:
            return False
        return cls.log_file_ino[path]['ino_changed']

    @classmethod
    def reset_ino_changed(cls, path=None):
        cls.log_file_ino[path]['ino_changed'] = False

    def monitor(self, replace=False):
        # 线程替换时不做增量统计
        if not replace:
            self.increment_thread_counter(1)
        # 如果被监控的文件不存在时,则一直循环尝试打开
        while True:
            try:
                with open(self.log_path, 'rU') as f:
                    offset = self.get_cursor(log_path=self.log_path)
                    if offset > 0:
                        f.seek(offset)
                    while True:
                        if self.get_exit_flag():
                            self.increment_thread_counter(-1)
                            return

                        self.line = f.readline()
                        if self.get_cursor(log_path=self.log_path) == f.tell():
                            if self.get_ino_changed(self.log_path):
                                self.reset_ino_changed(self.log_path)
                                kr = KeepRead()
                                kr.log_path = self.log_path
                                KeepRead.cursor[kr.log_path] = 0
                                return thread.start_new_thread(kr.monitor, (True, ))

                            time.sleep(1)
                            continue

                        self.set_cursor(log_path=self.log_path, offset=f.tell())
                        self.dispose()

            except IOError:
                time.sleep(1)
            except:
                self.increment_thread_counter(-1)
                raise

    def dispose(self):
        KeepRead.filtrator(self.line)

    @classmethod
    def signal_handle(cls, signum=0, frame=None):
        cls.exit_flag = True

    @staticmethod
    def launch():
        # ml for monitor log
        signal.signal(signal.SIGTERM, KeepRead.signal_handle)
        signal.signal(signal.SIGINT, KeepRead.signal_handle)
        KeepRead.load_config()
        KeepRead.restore_cursor()
        thread.start_new_thread(KeepRead.watch_ino, ())
        for item in KeepRead.config:
            kr = KeepRead()
            kr.log_path = item['path']

            if 'start_position' in item:
                KeepRead.cursor[kr.log_path] = item['start_position']
            elif kr.log_path not in KeepRead.cursor:
                KeepRead.cursor[kr.log_path] = 0

            thread.start_new_thread(kr.monitor, ())

        # 避免上面的for语句过快,导致线程还没有执行increment_thread_counter,
        # 就进入下面的while语句,这样thread_counter此时还为0,故而会跳过线程等待逻辑
        time.sleep(1)
        # 变通的方式实现线程等待目的
        while KeepRead.thread_counter > 0:
            time.sleep(1)
        KeepRead.save_cursor()
        print 'Bye-bye!'

